@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of the python-chess library.
-# Copyright (C) 2012-2014 Niklas Fiekas <niklas.fiekas@tu-clausthal.de>
+# Copyright (C) 2012-2015 Niklas Fiekas <niklas.fiekas@tu-clausthal.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,9 +16,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import chess
 import collections
 import signal
@@ -440,11 +437,7 @@ class GoCommand(Command):
         builder = []
         builder.append("go")
 
-        if searchmoves:
-            builder.append("searchmoves")
-            for move in searchmoves:
-                builder.append(move.uci())
-
+        self.ponder = ponder
         if ponder:
             builder.append("ponder")
 
@@ -488,6 +481,11 @@ class GoCommand(Command):
         if infinite:
             builder.append("infinite")
 
+        if searchmoves:
+            builder.append("searchmoves")
+            for move in searchmoves:
+                builder.append(move.uci())
+
         self.buf = " ".join(builder)
 
     def execute(self, engine):
@@ -495,7 +493,7 @@ class GoCommand(Command):
         engine.ponder = None
         engine.bestmove_received.clear()
         engine.send_line(self.buf)
-        if self.infinite:
+        if self.infinite or self.ponder:
             self.set_result(None)
         else:
             engine.bestmove_received.wait()
@@ -519,10 +517,15 @@ class StopCommand(Command):
         self.set_result(BestMove(engine.bestmove, engine.ponder))
 
 
-class PonderhitCommand(IsReadyCommand):
+class PonderhitCommand(Command):
     def execute(self, engine):
+        engine.bestmove = None
+        engine.ponder = None
+        engine.bestmove_received.clear()
         engine.send_line("ponderhit")
-        super(PonderhitCommand, self).execute(engine)
+
+        engine.bestmove_received.wait()
+        self.set_result(BestMove(engine.bestmove, engine.ponder))
 
 
 class QuitCommand(Command):
@@ -607,6 +610,7 @@ class PopenProcess(object):
     def __init__(self, command):
         self.command = command
 
+        self.process = None
         self._receiving_thread = threading.Thread(target=self._receiving_thread_target)
         self._receiving_thread.daemon = True
 
@@ -663,6 +667,9 @@ class SpurProcess(object):
 
         self._result = None
 
+        self.process = None
+        self.spawned = threading.Event()
+
         self._waiting_thread = threading.Thread(target=self._waiting_thread_target)
         self._waiting_thread.daemon = True
 
@@ -670,9 +677,15 @@ class SpurProcess(object):
         self.engine = engine
 
         self.process = self.shell.spawn(self.command, store_pid=True, allow_error=True, stdout=self)
+        self.spawned.set()
+
         self._waiting_thread.start()
 
     def write(self, byte):
+        # Wait for spawn to return. Otherwise we might already try processing
+        # data before self.process is set.
+        self.spawned.wait()
+
         # Interally called whenever a byte is received.
         if byte == b"\r":
             pass
@@ -823,11 +836,17 @@ class Engine(object):
             info_handler.pre_bestmove(arg)
 
         tokens = arg.split(None, 2)
-        self.bestmove = chess.Move.from_uci(tokens[0])
+
+        if tokens[0] != "(none)":
+            self.bestmove = chess.Move.from_uci(tokens[0])
+        else:
+            self.bestmove = None
+
         if len(tokens) >= 3 and tokens[1] == "ponder" and tokens[2] != "(none)":
             self.ponder = chess.Move.from_uci(tokens[2])
         else:
             self.ponder = None
+
         self.bestmove_received.set()
 
         for info_handler in self.info_handlers:
@@ -1145,7 +1164,7 @@ class Engine(object):
 
         Instead of just the final FEN, the initial FEN and all moves leading
         up to the position will be sent, so that the engine can detect
-        repititions.
+        repetitions.
 
         If the position is from a new game it is recommended to use the
         *ucinewgame* command before the *position* command.
@@ -1165,7 +1184,8 @@ class Engine(object):
         so that the engine knows how long to calculate.
 
         :param searchmoves: Restrict search to moves in this list.
-        :param ponder: Bool to enable pondering mode.
+        :param ponder: Bool to enable pondering mode. The engine will not stop
+            pondering in the background until a *stop* command is received.
         :param wtime: Integer of milliseconds white has left on the clock.
         :param btime: Integer of milliseconds black has left on the clock.
         :param winc: Integer of white Fisher increment.
@@ -1182,8 +1202,9 @@ class Engine(object):
         :return: **In normal search mode** a tuple of two elements. The first
             is the best move according to the engine. The second is the ponder
             move. This is the reply expected by the engine. Either of the
-            elements may be *None*. **In infinite search mode** there is no
-            result. See *stop* instead.
+            elements may be *None*. **In infinite search mode** or
+            **ponder mode** there is no result. See *stop* (or *ponderhit*)
+            instead.
         """
         return self._queue_command(GoCommand(searchmoves, ponder, wtime, btime, winc, binc, movestogo, depth, nodes, mate, movetime, infinite), async_callback)
 
@@ -1204,7 +1225,9 @@ class Engine(object):
         The engine should continue searching but should switch from pondering
         to normal search.
 
-        :return: Nothing
+        :return: A tuple of two elements. The first element is the best move
+        according to the engine. The second is the new ponder move. Either
+        of the elements may be *None*.
         """
         return self._queue_command(PonderhitCommand(), async_callback)
 
